//
//	DragDemo
//
//	by Tim Tomkinson
//	1/29/95
//
//	This program demonstrates how to drag a window correctly (and not so correctly).  If the
//	window's title bar is clicked, the window is dragged in the standard way.  If the content
//	region of the window is clicked, the window is dragged in "real-time", meaning that the
//	window itself is moved with the cursor, not just a dotted outline of the window, while the
//	background continuously updates itself.
//
//	In addition to window dragging, it demonstrates other common Macintosh tasks, i.e. setting
//	up a basic event loop, drawing the menu bar, displaying a picture in a window, handling
//	update events, etc.
//
//	You can use your own picture by replacing PICT resource 128, or by adding a new picture
//	and changing the constant "kPictureID" below.  You may need to increase the memory partition
//	size if you use a larger picture.
//
//	This file is formatted in Times 12, tab stops 4.
//
//	This program is in the public domain.  Please send correspondence to "ttomkinson@aol.com".
//


//	Constants

#define kWindowID	128				//  the window's resource ID
#define kAboutID		128				//  the About box's resource ID
#define kPictureID		128				//  the picture's resource ID
#define kAboutPicID	129				//  the About box's picture resource ID
#define kMenuBarID	128				//  the menu bar's resource ID
#define kAppleMenuID	128				//  the Apple menu's resource ID
#define kFileMenuID	129				//  the File menu's resource ID
#define kMaxWait		0x7FFFFFFF		//  wait time for "WaitNextEvent()"

//	Global variables

PicHandle	gPicture;				//  the picture
WindowPtr	gWindow;			//  the window
Boolean		gDone = FALSE;		//  program termination flag
EventRecord	gTheEvent;			//  event record


void ToolboxInit (void)

//	This function initializes the Macintosh Toolbox.  It should be called at the beginning of every program.

{
	InitGraf (&thePort);				//  initializes QuickDraw
	InitFonts ();					//  initializes the Font Manager
	FlushEvents (everyEvent, 0);		//  removes all pending events from the event queue
	InitWindows ();				//  initializes the Window Manager
	InitMenus ();					//  initializes the Menu Manager
	TEInit ();						//  initializes TextEdit
	InitDialogs (NULL);			//  initializes the Dialog Manager.
	InitCursor ();					//  sets the cursor to an arrow
	MaxApplZone ();				//  expands the application heap zone to its limit
}


void MenuInit (void)

//	This function reads in the menu bar from the resource file, adds the Apple menu items, then displays it.

{
	//	Read in the menu bar from the resource file and make it the current menu.  Add the desk accessories
	//	to the apple menu, then draw the menu bar.
	
	SetMenuBar (GetNewMBar (kMenuBarID));
	AddResMenu (GetMHandle (kAppleMenuID), 'DRVR');
	DrawMenuBar ();
}


void DrawWindow (void)

//	This function loads the picture and window from the resource file, then draws the window.  The
//	picture will be drawn later in response to the initial update event.

{
	//	Read in the picture and move its frame so that the top-left corner is (0,0).  This makes it easier
	//	to determine the picture's height and width.
	
	gPicture = GetPicture (kPictureID);
	OffsetRect (&(**gPicture).picFrame, -(**gPicture).picFrame.left, -(**gPicture).picFrame.top);
	
	//	Read in the window template, size the window to the size of the picture, show the window,
	//	then make it the current grafPort.
	
	gWindow = GetNewCWindow (kWindowID, NULL, (WindowPtr) -1);
	SizeWindow (gWindow, (**gPicture).picFrame.right, (**gPicture).picFrame.bottom, TRUE);
	ShowWindow (gWindow);
	SetPort (gWindow);
}


void UpdateWindow (void)

//	This function is called in response to an update event.  It simply draws the picture.  "BeginUpdate()"
//	and "EndUpdate()" must always be called when responding to an update event.  They handle the update
//	region so that only the part of the window that changed will be redrawn.

{
	BeginUpdate (gWindow);
	DrawPicture (gPicture, &(gWindow->portRect));
	EndUpdate (gWindow);
}


void MyDragWindow ()

//	This is the custom drag routine to drag the window in "real-time".

{
	Point	initialPoint,			//  the initial point where the mouse was clicked
			currentPoint,			//  the current mouse position
			lastPoint;				//  the previous mouse position
	Rect		dragRect;				//  the area where the window can be dragged
	
	//	Calculate the area where the window can be dragged.  This is 4 pixels from the edge of the
	//	screen.  The height of the menu bar is subtracted from the top edge to prevent the window
	//	from being moved under it.
	
	dragRect = qd.screenBits.bounds;
	InsetRect (&dragRect, 4, 4);
	dragRect.top += GetMBarHeight ();

	//	Calculate the point where the mouse was initially clicked (in the window's coordinate system).
	
	initialPoint = gTheEvent.where;
	GlobalToLocal (&initialPoint);

	//	Loop while the mouse button is held down.
	
	while (StillDown ())
	{
		//	Get the current mouse position, in global coordinates.
		
		GetMouse (&currentPoint);
		LocalToGlobal (&currentPoint);
		
		//	If the point is inside the drag rectangle, then the window can be moved.
		
		if (PtInRect (currentPoint, &dragRect))
		{
			//	Subtract the initial point from the current point.  This will cause the window to be
			//	moved with the cursor, regardless of where in the window it was clicked.
			
			SubPt (initialPoint, &currentPoint);
			
			//	If the current point is different from the last point, move the window and reset the
			//	last point.  This check is performed so that "MoveWindow()" is not called when the
			//	mouse is not moving (which would cause the cursor to blink annoyingly).
			
			if (!EqualPt (currentPoint, lastPoint))
			{
				MoveWindow (gWindow, currentPoint.h, currentPoint.v, FALSE);
				lastPoint = currentPoint;
			}
		}
		
		//	Call "WaitNextEvent()" to give background applications a chance to update their windows.
		
		WaitNextEvent (everyEvent, &gTheEvent, kMaxWait, NULL);
		
		//	If an update event is received, the window needs to be redrawn.
		
		if (gTheEvent.what == updateEvt)
			UpdateWindow ();
	}
}


void HandleAppleChoice (short theItem)

//	Handles a choice from the Apple menu, where "theItem" is the menu item number.

{
	Str255 deskAccessory;		//  name of desk accessory

	//	If the "About DragDemo" item was chosen, display the alert box.
	
	if (theItem == 1)
		Alert (kAboutID, NULL);

	//	Otherwise, get the name of the desk accessory chosen and open it.
	
	else
	{
		GetItem (GetMHandle (kAppleMenuID), theItem, &deskAccessory);
		OpenDeskAcc (deskAccessory);
	}
}


void HandleMenuChoice (long menuChoice)

//	Handles a menu choice.

{
	//	Check the menu ID, which is stored in the high-order word of "menuChoice".
	
	switch (HiWord (menuChoice))
	{
		//	If the Apple menu was selected, call "HandleAppleChoice" with the menu item number,
		//	which is stored in the low-order word of "menuChoice".
		
		case kAppleMenuID:
			HandleAppleChoice (LoWord (menuChoice));
			break;
		
		//	If an item in the file menu was selected, set the program termination flag (since "Close"
		//	and "Quit" do the same thing, it doesn't matter which one was selected).
		
		case kFileMenuID:
			gDone = TRUE;
			break;
	}
	
	//	"HiliteMenu()" must be called to unhilite the menu bar.
	
	HiliteMenu (0);
}


void HandleMouseDown (void)

//	Handles a mouse down event.

{
	WindowPtr whichWindow;		//  which window was clicked

	//	Determine where and in which window the mouse was clicked.  It is not really necessary
	//	to determine which window was clicked in this program, since there is only one window.
	
	switch (FindWindow (gTheEvent.where, &whichWindow))
	{
		case inMenuBar:
	
			//	If the menu bar was clicked, call "MenuSelect()" to determine the menu choice
			//	and pass it to "HandleMenuChoice()".
		
			HandleMenuChoice (MenuSelect (gTheEvent.where));
			break;

		case inGoAway:
			
			//	If the go-away (close) box was clicked, call "TrackGoAway()" to track the mouse and
			//	determine if it was released inside the box.  If so, set the termination flag.

			if (TrackGoAway (gWindow, gTheEvent.where))
				gDone = TRUE;
			break;
			
		case inDrag:

			//	If the drag region (title bar) was clicked, call "DragWindow()" to drag the window in
			//	the normal fashion.
		
			DragWindow (gWindow, gTheEvent.where, &qd.screenBits.bounds);
			break;
			
		case inContent:
		
			//	If the content area was clicked, call the custom drag routine "MyDragWindow()".
			
			MyDragWindow ();
			break;
	}
}


void HandleEvents (void)

//	This function repeatedly handles events until the "gDone" flag is set (by closing the window, or
//	selecting "Close" or "Quit" from the File menu).

{
	do
	{
		//	Call "WaitNextEvent()" to get the next event.
		
		WaitNextEvent (everyEvent, &gTheEvent, kMaxWait, NULL);
		switch (gTheEvent.what)
		{
			case mouseDown:
			
				//	If the mouse was clicked, call "HandleMouseDown()".
				
				HandleMouseDown ();
				break;
				
			case keyDown:
			
				//	If a key was pressed, check to see if the command key was also pressed.  If so,
				//	mask out the character from the event message, then call "MenuKey()" to determine
				//	the menu choice and send it to "HandleMenuChoice()".
				
				if (gTheEvent.modifiers & cmdKey)
					HandleMenuChoice (MenuKey (gTheEvent.message & charCodeMask));
				break;
				
			case updateEvt:
			
				//	If an update event was received, call "UpdateWindow()" to redraw the picture.
				
				UpdateWindow ();
				break;
		}
	}
	while (!gDone);
}


void main (void)

//	The main function initializes everything, then calls "HandleEvents()" to process events until the
//	user quits.

{
	ToolboxInit ();			//  initialize the Macintosh Toolbox
	MenuInit ();			//  draw the menu bar
	DrawWindow ();		//  draw the window
	HandleEvents ();		//  handle events
}
